package PiTube::Controller::Calendar;

use feature 'signatures';
no warnings qw{ experimental::signatures };
use Mojo::Base 'Mojolicious::Controller';
use Mojo::UserAgent;

sub list {
    my $c = shift;

    my $ua = Mojo::UserAgent->new;

    my $res;

    eval { $res = $ua->get( $c->config->{calendar}{iapi} )->result; };

    # TODO: OpenAPI error
    if ( $@ || ! $res->is_success ) {

        $c->app->log->error( $@ ? $@ : $res->message);

        $c->render( json => {
            total => 0,
            rows  => [],
        });
        return;
    }

    my @calendar;
    my @days = sort { $a->{date} cmp $b->{date} } @{ $res->json->{days} };

    DAY:
    foreach my $day ( @days ) {
        my @events = sort { $a->{start} cmp $b->{start} } @{ $day->{events} };

        EVENT:
        foreach my $event ( @events ) {

            # cistka nejhakeho bordelu z iCal. asi radsi v iAPI
            $event->{summary} =~ s/\\\W/$1/g;

            my $item = {
                date        => $day->{date_cz},
                time        => $event->{start} . '-' . $event->{end},
                description => $event->{summary},
            };

            if (
                $event->{summary} =~ s/^\s*stream\s+(\w+)\W+//i
                ||
                $event->{summary} =~ s/^\s*(\w+)\W+//i
            ) {
                my $key = lc($1);

                my $stream = $c->schema->resultset('Stream')->search({
                    key => $key
                })->first;

                if ( $stream ) {
                    $item->{description} = ucfirst($event->{summary});
                    $item->{stream_name} = $stream->name;
                    if ( $stream->is_live ) {
                        $item->{stream_key}  = $stream->key;
                    }
                }
            }

            push @calendar, $item;
        }
    }

    $c->render( json => {
        total => scalar @calendar,
        rows  => \@calendar,
    });

};

1;
