package PiTube::Controller::Stream;

use feature 'signatures';
no warnings qw{ experimental::signatures };
use Mojo::Base 'Mojolicious::Controller';
use Mojo::UserAgent;
use Mojo::Pg::PubSub;
use Data::Random qw(rand_chars);

use constant CONTENT_TYPE => {
    m3u8 => 'application/vnd.apple.mpegurl',
    ts   => 'video/mp2t',
};

use constant CONTROL_URL => '%s/control/record/%s?app=stream&name=%s&rec=%s';
use constant HLS_URL     => '/hls/%s.m3u8';

sub list {
    my $c = shift;

    my $readable = $c->schema->resultset('ACL')->user_acl(
        $c->session->{user}, 4
    );

    my $cond = {
        is_active => 't',
        -or => [
            { is_public => 't' },
            { key => { -in => $readable }},
        ]
    };

    $cond->{is_live} = 't' if $c->param('live');

    my $streams = $c->schema->resultset('Stream_view')->search(
        $cond,
        { order_by => 'name' }
    );

    my @streams = ();

    STREAM:
    while ( my $stream = $streams->next()) {

        my $rights = $c->session->{user}{acl}{ $stream->key };

        my %stream = (
            $stream->get_columns(),
            is_writeable  => $stream->is_writeable( $rights ),
            is_recordable => $stream->is_recordable( $c->session->{user}{id} ),
        );

        delete $stream{publish_user_name} if ! $stream->is_live;

        push @streams, \%stream;
    }

    $c->render( json => \@streams );
}

sub player {
    my $c = shift;

    # vzdy aktualizovat
    $c->session->{user}{acl} = $c->schema->resultset('ACL')->user_acl(
        $c->session->{user}
    );

    $c->session->{user}{wid} ||= rand_chars( set => 'alphanumeric', size => 16 );

    # stream
    my $stream = $c->schema->resultset('Stream_view')->find({
        key => $c->stash->{key}
    });

    if ( ! $stream ) {
        $c->render('stream/404');
        return;
    }

    $c->stash->{stream} = {
        $stream->get_columns,
        is_live => $stream->is_live,
    };

    if ( ! ( $c->session->{user}{acl}{ $stream->key } & 4 ) ) { # TODO:constant
        $c->render('stream/403');
        return;
    }

    $c->stash->{url} = sprintf(HLS_URL, $stream->key);

    $c->render();
}

sub info {
    my $c = shift;

    # vzdy aktualizovat
    $c->session->{user}{acl} = $c->schema->resultset('ACL')->user_acl(
        $c->session->{user}
    );

    # stream
    my $stream = $c->schema->resultset('Stream_view')->find({
        id => $c->stash->{id}
    });

    if ( ! $stream ) {
        $c->render('stream/404');
        return;
    }

    my $rights = $c->session->{user}{acl}{ $stream->key };

    if ( ! $rights ) {
        $c->render('stream/403');
        return;
    }

    $c->stash->{stream} = {
        $stream->get_columns(),
        is_writeable  => $stream->is_writeable( $rights ),
        is_recordable => $stream->is_recordable( $c->session->{user}{id} ),
    };

    $c->render();
}


1;

__END__

