package PiTube::Schema::Result::Record;

use strict;
use warnings;
use feature 'signatures';
no warnings qw{ experimental::signatures };

use base 'DBIx::Class::Core';

our $VERSION = 1;

__PACKAGE__->table('records');

__PACKAGE__->add_columns(
    id => {
        data_type         => 'integer',
        is_auto_increment => 1,
        is_nullable       => 0,
        sequence          => 'uid_seq'
    },
    qw(
        begin
        end
        is_active
        is_indexed
        is_protected
        stream_id
        publish_user_id
        recorder
        path
        description
        info
    ),
);

__PACKAGE__->belongs_to(
    stream => 'PiTube::Schema::Result::Stream',
    {
        'foreign.id' => 'self.stream_id',
    },
);
__PACKAGE__->set_primary_key('id');

sub is_editable ($self, $user_id=0) {
    return $user_id == $self->publish_user_id;
}

sub is_deletable ($self, $user_id=0) {

    return 0 if $self->is_protected;
    return $user_id == $self->publish_user_id;
}

1;
