package PiTube::Schema::ResultSet::ACL;

use strict;
use warnings;

use base 'DBIx::Class::ResultSet';

sub user_acl {
    my $class = shift;
    my $user  = shift;
    my $right = shift // 0;
    my $rights = {};

    # verejne streamy

    my $streams = $class->result_source->schema->resultset('Stream')->search(
        { is_public => 't' }
    );

    STREAM:
    while ( my $stream = $streams->next ) {
        $rights->{ $stream->key } = 4;
    }

    # na zaklade clenstvi ve skupine, role nebo user.id

    my $acls = $class->search(
        {
            -or => [
                {
                    class => 'all',
                },
                {
                    class => 'group',
                    value => { '-in' => $user->{groups} // [] },
                },
                {
                    class => 'role',
                    value => { '-in' => $user->{roles} // [] },
                },
                {
                    class => 'user',
                    value => $user->{id} // '',
                },
            ]
        },
        {
            join => 'stream'
        }
    );

    ACL:
    while ( my $acl = $acls->next ) {
        $rights->{ $acl->stream->key } |= $acl->rights;
    }

    if ( $right ) {
        my @keys = ();

        KEY:
        foreach my $key ( keys %{ $rights } ) {
            if ( $rights->{$key} & $right ) {
                push @keys, $key;
            }
        }
        return \@keys;
    }

    return $rights;

}


1;

__END__

